#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "dArray.hpp"
#include "Evts.hpp"
#include "regexp.hpp"

#define VERSION		"v.00.00alpha"
typedef int		boolean;
typedef unsigned short	ushort;
#define numberof(arr) (sizeof(arr)/sizeof(arr[0]))

#define UNDEFINED_NODE	((ushort)(-1))

struct ndlStr {
static ushort		currZone;
static ushort		currNet;

	char 		Prefix[10];
	ushort		Number;
	char		System[100];
	char		Location[100];
	char		Sysop[100];
	char		Phone[30];
	long		Speed;
	int		nFlags;
	char 		Flags[100];

	boolean		parse(char * str);
	void		print(void);
};

struct storedNode {
	char 		Prefix[10];
	ushort		Number;
	char		Phone[30];

	storedNode(char * pr, ushort n, char * ph);
};

class Ndl: public dynArr<storedNode> {
public:
	Ndl(char * path);
	boolean		check();
};

class Implied {
	RegExp 		r1;		// first flag
	RegExp 		r2;		// implied
	char		reason[100];	// diagnostics

public:
	Implied(char * t1, char * t2, char * r):
		r1(t1),r2(t2){strncpy(reason,r,99);}
	Implied(Implied& i):
		r1(i.r1),r2(i.r2){strncpy(reason,i.reason,99);}
	boolean test(char * flags) {
		if (r1.test(flags))
			return !r2.test(flags);
		else
 			return 1;}
	char * Reason(void) {return reason;}
};


struct legalPrefix {
	size_t		length;
	RegExp 		Prefix;
	legalPrefix(legalPrefix& l):Prefix(l.Prefix){length=l.length;}
	legalPrefix(char * r, size_t l):Prefix(r){length=l;}
	boolean test(char * phone) { 
		if (length && strlen(phone) != length) return 0;
		else return Prefix.test(phone); }
};

class NetCfg {
public:
	ushort		Number;
private:
	dynArr<ushort>		Pvts;
	dynArr<legalPrefix>	Prfxs;

public:
	NetCfg(ushort num = UNDEFINED_NODE) : Pvts(10,5), Prfxs(5,5)
			{Number = num;}
	void	addPvt(ushort n) { Pvts.add(new ushort(n)); }
	void	addPrfx(char * r, size_t l)
		{ Prfxs.add(new legalPrefix(r,l)); }
	boolean	testPvt(ushort n);
	boolean	testPhone(char * ph);
};

class Config {
	int		currLine;
	char *		currFile;

	char *		strFlags;
	RegExp *	possibleFlags;
	dynArr<Implied>	Implies;
	dynArr<NetCfg>	Net;

	void	parse(char * cfgname);
	void	dive2Include(char * p);
	void	loadFlags(char *);
	void	loadImplies(char *);
	void	loadNetwork(char *);
	void	loadPvt(char *);
public:
	Config(char * cfgname);

	boolean testFlag(char * f){return possibleFlags->test(f);}
	boolean testImplies(char * f) { int i;
		for (i = 0; i < Implies.nItems(); i++)
			if (!Implies[i].test(f)) {
			toLog(E_IMPLIED,Implies[i].Reason()); return 0; }
		return 1;}
	boolean testLegalPvt(ndlStr&);
	boolean testPhone(ndlStr&);
	NetCfg * getNetCfg(ushort n) { int i;
		for (i = 0; i < Net.nItems(); i++)
			if (Net[i].Number == n) return &Net[i];
		return NULL; }
	parse(FILE * fil);
};


inline void skipws(char **p) 
{ while (**p == ' ' || **p == '\t' || **p == '\r') (*p)++;}

// something like BSD4.4 strsep(), but more flexible
// and leave the string unchanged
// do not use one tokenizer object for simultanious parsing
// more than one string
class tokenizer {
	char *	d;		// delimiters
	char *	s;		// stop bytes
	boolean	skipseq;	// skip one or more subsequental delims
	boolean	d0;		// 0 is delimiter
	boolean	s0;		// 0 is stop

	char *	cfirst;		// 1st position of field
	char *	clast;		// last+1 position

	char * walk(char * to = NULL, size_t to_len = 0);
public:
	tokenizer(char * const delim, char * const stop, 
		boolean ss = 1, boolean del0 = 0, boolean st0 = 1) :
		skipseq(ss), d0(del0),s0(st0){ d = delim; s = stop;}

	char * first(char * str, char * to = NULL, size_t to_len = 0) {
		cfirst = clast = str; return walk(to,to_len); }
	char * next(char * to = NULL, size_t to_len = 0);
	char * last(void) const {return clast;}
};
