#include "ChkNl.hpp"

static RegExp possiblePrefixes("^(Zone)?|(Region)|(Host)|(Hub)|(Hold)|(Down)|(Pvt)$");

Config * cfg = NULL;
ushort ndlStr::currNet = 50;
ushort ndlStr::currZone = 2;

void toLog(int code, char * reason) {
	printf("***Error %02d -- %s\n", code, reason);
}

static RegExp	rdigs("^[0-9]+$"),
		rname("^[^ ]+$"),
		rsysop(rname),
		rflag("^[0-9a-zA-Z#!:@_\.\-]+$"),
		rphone("^((-|[0-9])+|(\-Unpublished\-))$");

boolean ndlStr::parse(char * input) {
	char	tmp[200];
	long	ltmp;
	int	uFlags = 0;

	Number = UNDEFINED_NODE;

	tokenizer t(",","\n",0);


	if (*t.first(input,tmp,200) == ';')
		return 1;		// Comment -- OK

	if (!possiblePrefixes.test(tmp))	// *** Prefix
		toLog(E_SYNTAX, "prefix");
	else
		strcpy(Prefix, tmp);

	t.next(tmp,200);		// *** Node number
	if (!rdigs.test(tmp) || (ltmp = atol(tmp)) > 65535l)
		toLog(E_SYNTAX, "node number");
	else {
		Number = 0;
		if (strcmp(Prefix, "Zone") == 0) {
			ndlStr::currNet = (ushort)ltmp;
			ndlStr::currZone = (ushort)ltmp;
		} else if (strcmp(Prefix, "Region") == 0)
			ndlStr::currNet = (ushort)ltmp;
		else if (strcmp(Prefix, "Host") == 0)
			ndlStr::currNet = (ushort)ltmp;
		else
			Number = (ushort)ltmp;
	}
	t.next(tmp,200);		// *** System
	if (!rname.test(tmp))
		toLog(E_SYNTAX,"system name");
	else
		strcpy(System, tmp);

	t.next(tmp,200);		// *** Location
	if (!rname.test(tmp))
		toLog(E_SYNTAX, "location");
	else
		strcpy(Location, tmp);

	t.next(tmp,200);		// *** Sysop
	if (!rsysop.test(tmp))
		toLog(E_SYNTAX, "sysop name");
	else
		strcpy(Sysop, tmp);

	t.next(tmp,200);		// *** Phone
	if (!rphone.test(tmp)) {
		toLog(E_SYNTAX, "phone number");
	} else {
		if (strcmp(tmp, "-Unpublished-") == 0) {
			if (strcmp(Prefix, "Pvt") != 0)
				toLog(E_SYNTAX, "-Unpublished- but not Pvt");
		} else {
			int cnt = 0, cnt1 = 0;
			while (tmp[cnt]) {
				if (tmp[cnt++] != '-')
					Phone[cnt1++] = tmp[cnt-1];
			}
			Phone[cnt1] = '\0';
			if (!cfg->testPhone(*this))
				toLog(E_ILLPHONE, "Illegal phone");
		}
	}

	t.next(tmp,200);		// *** Baud rate
	if (!rdigs.test(tmp))
		toLog(E_SYNTAX, "baud rate");
	else
		Speed = atol(tmp);

					// *** flags ***
	uFlags = 0;
	Flags[0] = '\0';
	while (*t.last() != '\n' && *t.last() != '\0') {
		tmp[0] = 'U';
		t.next(tmp+1,200);
		if(!rflag.test(tmp)) {
			toLog(E_SYNTAX, tmp+1);
		} else {
			if (tmp[1] == 'U') {
				uFlags = 1;
			} else {
				if (!cfg->testFlag(tmp+1-uFlags)) {
					toLog(E_ILLFLAG, tmp+1-uFlags);
				} else {
					strcat(Flags, " ");
					strcat(Flags, tmp+1-uFlags);
				}
			}
		}
	}
	strcat(Flags, " ");
	if (!cfg->testImplies(Flags))
		return 0;
	if (strcmp(Prefix, "Pvt") == 0 && !cfg->testLegalPvt(*this)) {
		toLog(E_ILLPVT, "");
		return 0;
	}

	return 1;	
}

void ndlStr::print(void) {
	printf("%s\n%d\n%s\n%s\n%s\n%s\n%ld\n%s\n",
		Prefix, Number, System, Location, Sysop, Phone, Speed,Flags);
}

void
regerr( char* s)
{
	fprintf(stderr, "regexp(3): %s", s);
	exit(1);
}


int
main(int argc, char * argv[]) {
	
	char buff[200];
	ndlStr str;
	
	set_regerror_func(regerr);

	fprintf(stderr, "CheckNdl "VERSION" (c) SfY 1999\n");
cfg = new Config("ChkNl.cfg");

	while (fgets(buff,200,stdin) != NULL) {
		printf("\n\n%s",buff);
		str.parse(buff);
//		str.print();
	}	
	return 0;
}
