//
// ChkNdl config parser (c) SfY 1999-2000
//

#include "ChkNl.hpp"

static void loadChoices(char * dest, char * const src, size_t strtup) {
	tokenizer t(" \t",";\n");
	char 	*rc, *p;
	boolean thefirst = (strlen(dest) == strtup);
	p = dest+strlen(dest);

	for (rc = t.first(src); rc != NULL; rc = t.next()) {
		if (!thefirst) *(p++) = '|';
		thefirst = 0; *(p++) = '(';
		while (rc < t.last()) *(p++) = *(rc++);
		*(p++) = ')';
	}
	*p = '\0';
}

Config::Config(char * name) : Implies(20,10), Net(100,50) {
	strFlags = (char *)malloc(4000);
	if (strFlags == NULL) {
		toLog(E_FATAL, "no memory strFlag");
	}
	strcpy(strFlags,"^(");
	parse(name);
	strcat(strFlags,")$");
	possibleFlags = new RegExp(strFlags);
	free(strFlags);
}

void Config::parse(char * name) {
	FILE * input;

	char	buff[400];
	char *	p = buff;

	currFile = name;
	currLine = 0;

	if ((input = fopen(name,"r")) == 0) {
		toLog(E_NOCONFIG, name);
		return;
	}

	while(fgets(buff,400,input) != NULL) {
		currLine++;
		skipws(&p);
		if (strnicmp(p,";",1) == 0)
			;
		else if (strnicmp(p, "include", 7) == 0) {
			dive2Include(p+7);
		} else if (strnicmp(p, "flags", 5) == 0) {
			loadFlags(p+5);
		} else if (strnicmp(p, "implies", 7) == 0) {
			loadImplies(p+7);
		} else if (strnicmp(p, "network", 7) == 0) {
			loadNetwork(p+7);
		} else if (strnicmp(p, "legal_pvt", 9) == 0) {
			loadPvt(p+9);
		}
	}

}

char * getPath(char * p) {
	tokenizer	t(" \t","\n");
	char * p1 = t.first(p);
	*t.last()='\0';
	return p1;
}

void Config::dive2Include(char * p) {
	skipws(&p);
	char *	f = currFile;
	int	l = currLine;
	parse(getPath(p));
	currFile = f;
	currLine = l;
}

void Config::loadFlags(char * p) {
	loadChoices(strFlags, p, 2);
}

void Config::loadNetwork(char * p) {
	tokenizer	t(" \t", "\n");
	RegExp	d("^[0-9]+$");
	char 	buff[200];
	size_t	l;

	skipws(&p);
	t.first(p,buff,50);

	if (!d.test(buff)) {
		toLog(E_CONFIG, "network number");
		return;
	}
	ushort n = (ushort)atol(buff);
	NetCfg * ncfg;
	if ((ncfg = getNetCfg(n)) == NULL)
		if(!Net.add(new NetCfg(n))) {
			toLog(E_FATAL, "no memory NetCfg");
			return;
		} else
			ncfg = getNetCfg(n);
	t.next(buff,50);
	if (strcmp(buff,"-") == 0)
		l = 0;
	else {
		if (!d.test(buff)) {
			toLog(E_CONFIG, "phone length");
			return;
		} else
			l = (size_t)atol(buff);
	}
	strcpy(buff,"^(");
	p = t.last();
	skipws(&p);
	loadChoices(buff, p, 2);
	strcat(buff,")");
printf("%d -> %d %s\n",ncfg->Number, l, buff);
	ncfg->addPrfx(buff,l);
}

void Config::loadPvt(char * p) {
	tokenizer	t(" \t", "\n");
	RegExp	d("^[0-9]+$");
	char 	buff[50];
	skipws(&p);

	t.first(p,buff,50);
	if (!d.test(buff)) {
		toLog(E_CONFIG, "network number");
		return;
	}
	ushort n = (ushort)atol(buff);
	NetCfg * ncfg;
	if ((ncfg = getNetCfg(n)) == NULL)
		if(!Net.add(new NetCfg(n))) {
			toLog(E_FATAL, "no memory NetCfg");
			return;
		} else
			ncfg = getNetCfg(n);
	while (*t.last() != '\0' && *t.last() != '\n') {
		t.next(buff,50);
		if (!d.test(buff)) {
			toLog(E_CONFIG, "pvt_node number");
			return;
		} else
			ncfg->addPvt((ushort)atol(buff));
	}
}

void Config::loadImplies(char * p) {
	char		r1[200],r2[200];
	tokenizer t(" \t",";\n");
	skipws(&p);
	r1[0] = ' ';
	t.first(p,r1+1,200);
	strcat(r1, " ");

	t.next(r2,200);

	if (strcmp(r2,"=") != 0) {
		toLog(E_CONFIG, "implies");
		return;
	}
	r2[0] = '\0';
	int thefirst = 1;
	char * p1 = r2, * rc;
	for (rc = t.next(); rc != NULL; rc = t.next()) {
		if (!thefirst) *(p1++) = '|';
		thefirst = 0; *(p1++) = '(';*(p1++) = ' ';
		while (rc < t.last()) *(p1++) = *(rc++);
		*(p1++) = ' ';*(p1++) = ')';
	}
	*p1 = '\0';

	Implies.add(new Implied(r1,r2,p));
}

boolean Config::testLegalPvt(ndlStr & n) {
	if (strstr(n.Flags, "CM") != NULL && strstr(n.Flags, "CM") != NULL) {
		char buff[100];
		sprintf(buff,"f%d.n%d.z%d.fidonet.net$",
			n.Number, n.currNet, n.currZone);
		RegExp r(buff);
		if (r.test(n.System))
			return 1;
	}
	NetCfg * ncfg;
	if ((ncfg = getNetCfg(n.currNet)) != NULL)
		return ncfg->testPvt(n.Number);
	else
		return 0;
	
}

boolean	NetCfg::testPvt(ushort n) {
	int	i;
	for (i = 0; i < Pvts.nItems(); i++)
		if (Pvts[i] == n)
			return 1;
	return 0;
}

boolean	NetCfg::testPhone(char * ph) {
	int	i;
	for (i = 0; i < Prfxs.nItems(); i++)
		if (Prfxs[i].test(ph))
			return 1;
	return 0;
}

boolean	Config::testPhone(ndlStr & n) {
	NetCfg * ncfg;
	if ((ncfg = getNetCfg(n.currNet)) != NULL)
		return ncfg->testPhone(n.Phone);
	else
		return 0;
}

char * tokenizer::walk(char * to, size_t to_len) {
	char *	p = to;

	while (1) {
		if ((strchr(s,*clast)!=NULL || (s0 && *clast=='\0')) ||
		    (strchr(d,*clast)!=NULL && !(d0 && *clast=='\0'))) {
			if (to != NULL)
				*p = '\0';
			return cfirst;
		} else {
			if (to != NULL && p-to < to_len-2)
				*p++ = *clast++;
			else
				clast++;
		}
	}
}
char * tokenizer::next(char * to, size_t to_len) {
	cfirst = clast;
	do {
		if (strchr(s,*cfirst)!=NULL || (s0 && *cfirst=='\0')) {
			clast = cfirst;
			return NULL;
		}
		if (strchr(d,*cfirst)!=NULL || (d0 && *cfirst=='\0'))
			cfirst++;
		else
			break;
	} while (skipseq); // one pass if 0 and skip all delims if 1
	clast = cfirst;
	return walk(to, to_len);
}

